function fmri=CBFcalc(fmri, SIrefValue, cntRun, corrd3, R1factor)
%usage: fmri=CBFcalc(fmri, SIrefValue, cntRun, corrd3, R1factor)
%
% Calculates CBF values using appropriate equation
% Supported ASL acquisitions: PQ2, CSASL, PCASL
%
% Input:
%    fmri       - main structure used to get processing flags and data
%    SIrefValue - from CSF or WM
%    cntRun     - func run number
%    corrd3     - d3 data after min con correction
%    R1factor   - R1 relation between CSF and ASL (for BGS only)
%
% Output:
%    fmri fields CBFraw and CBF
%
% Notes:
%    - We are setting factor 'q' to unity; see Luh WM et al MRM 1999;
%      introduces only 3% error
%    - Not sure if this works for single slice data...
%
% Copyright (c) 2006 CUPID
% Author: jperthen
% History:
%  $Id: CBFcalc.m,v 1.4 2006/12/14 19:05:44 jperthen Exp $

% ruler --1234567890123456789012345678901234567890123456789012345678901234567890

% get essential info
study       = [fmri.dir.data,'/',fmri.study.name,'/'];
funcName    = fmri.func(cntRun).name(1,:);
funcbrikLoc = [study,funcName];
if strcmp(fmri.func(cntRun).params.pulseSeq, 'NaN')
    error('params.pulseSeq info required for CBF quantification!');
end
spepVer = str2num(fmri.func(cntRun).params.pulseSeq(1,5:end))


% set global constants
T1blood          = 1.664;    % Lu et al 2004 MRM
T2starblood      = 0.106;    % T2 star arterial blood; extrapolated from 1.5,4,7T values given by St Lawrence et al MRM 2005 (T2star venous blood =~32 ms)
T2blood          = 0.275;    % Stanisz et al MRM 2005;54;507

% determine which T2 to use (SE or GE) - only available for spep038.4 and above
GE=0;SE=0;
if spepVer > 38.35
    fmri.func(cntRun).params=parseAFNIhead(fmri.func(cntRun).params, funcbrikLoc,{'oppseq'});
    if isfield(fmri.func(cntRun).params, 'oppseq')
        if fmri.func(cntRun).params.oppseq == 1
            SE  = 1;
            T2b = T2blood;
        elseif fmri.func(cntRun).params.oppseq == 2
            GE  = 1;
            T2b = T2starblood;
        end
    else
        disp('Cannot determine sequence type (gradient or spin echo) - assuming gradient echo');
        GE  = 1;
        T2b = T2starblood;
    end
else
    disp('Cannot determine sequence type (gradient or spin echo) - assuming gradient echo');
    GE  = 1;
    T2b = T2starblood;
end

% Set T2 ref and ratio depending on SIref tissue and SE vs GE
% assume that the CSF image is always GRADIENT ECHO
if strcmp(fmri.func(cntRun).params.SIref,'WM')
    ratio       = 1.2279;            % blood:WM proton density ratio at TE=0. measured with spgr at TE=9.1ms: R=1.360
    %if GE ==1
    T2Ref       = 0.0484;        % T2starWM: OCCIPITAL value from Wansapura et al JMRI 1999
    %     elseif SE ==1
    %         T2Ref       = 0.110;         % T2WM: mean value from Wansapura et al JMRI 1999
    %     end
elseif strcmp(fmri.func(cntRun).params.SIref,'CSF')
    ratio       = 0.7212;      % blood:CSF proton density ratio at TE=0; measured with spgr at TE=9.1ms; R=0.672
    %if GE ==1
    T2Ref       = 0.6;         % T2starCSF : measured at cFMRI
    %     elseif SE ==1
    %         error('Need T2 value for CSF!!!!!');
    %         T2Ref       = ;
    %     end
end

% Set BGSfactor
BGSfactor=1.0;
if isfield(fmri.func(cntRun).params, 'bgs_on')  % later spep versions
    if fmri.func(cntRun).params.bgs_on          % BGS was used
        beep;
        BGSans   = input(['Background suppression was used for this run!! Do you want to continue (BGS factor not yet set) (y/[n])?'], 's');
        if strcmp(lower(BGSans), 'n')
            error('BGS factor required');
            %BGSfactor = ;
        end
    end
end

% scan-specific variables
if length(fmri.func(cntRun).params.TI)==2
    TI1        = fmri.func(cntRun).params.TI(1)*0.001;
    TI2        = fmri.func(cntRun).params.TI(2)*0.001;
else
    TI2        = fmri.func(cntRun).params.TI(1)*0.001;
    TI1=0;
end
TE         = fmri.func(cntRun).params.TE(1)*0.001;              % all times in seconds
sldelay    = fmri.func(cntRun).params.slDelay*0.001;   % time between acquisition of each slice - CV tmin

if (nnz(isnan([TE TI1 TI2 sldelay]))>0)
    error('TE, TI1, TI2 and slDelay are required for CBF calculation!');
end

% make the TI2 time slice-specific
sd3     = size(corrd3);
for m=0:sd3(3)-1
    TI2m(m+1)     = TI2+(m*sldelay);
end

% correct the SIrefValue for different R1s between CSF and ASL acquisitions
if R1factor>0
    SIrefValue = SIrefValue.*((sqrt(2))^R1factor);
end

% Get M0blood (NB: M0blood is M0 at the TE of the ASL acquisition (therefore no T2 terms in eqns below)
CSFbloodfactor    = exp(TE.*((1/T2Ref) - (1/T2b)));
M0blood           = SIrefValue.*ratio.*CSFbloodfactor;  % CSFbloodfactor cancels if TEspgr=TEasl. then M0blood = SIrefValue*ratioAtTE

% Do CBF calculation according to tagging scheme
ASLdata           = corrd3./BGSfactor;
CBFraw            = zeros(size(corrd3));
switch fmri.func(cntRun).params.tag
    case {2,3}  % PICORE Q2 or FAIR Q2
        if fmri.func(cntRun).params.tag==2
            disp('***************************************');
            disp('Quantifying FAIR QUIPSS 2 data');
            disp('***************************************');
        else
            disp('***************************************');
            disp('Quantifying PICORE QUIPSS 2 data');
            disp('***************************************');
        end
        alpha=1;        % assume inversion efficiency 100%
        for nn=1:sd3(3)
            expTerm          = exp(-TI2m(nn)./T1blood);
            CBFraw(:,:,nn,:) = (ASLdata(:,:,nn,:)./(2.*alpha.*M0blood.*TI1.*expTerm)).*6000;  % 6000 for correct units
        end

    case 4  % VSASL
        disp('***********************************');
        disp('Quantifying VELOCITY SELECTIVE data');
        disp('***********************************');
        if spepVer < 38.35
            error('Cannot quantify VAS-ASL data for this spep version - header info missing');
        end
        TR    = fmri.func(cntRun).params.TR;   % TR already in seconds. Read in during setup
        fmri.func(cntRun).params=parseAFNIhead(fmri.func(cntRun).params, funcbrikLoc,{'sechNSte'});
        TEtag = (2.*fmri.func(cntRun).params.sechNSte)./1000000;  % TEtag in s
        for nn=1:sd3(3)
            expTerm           = (exp(-TI2m(nn)/T1blood)).*(exp(-(TR-TI2m(nn))/T1blood)).*(exp(-TEtag/T2b));
            CBFraw(:,:,nn,:)  = (ASLdata(:,:,nn,:)./(TI2m(nn).*M0blood.*expTerm)).*6000;
        end

    case 9  % PCASL
        disp('***********************************');
        disp('Quantifying PSEUDO-CONTINUOUS data');
        disp('***********************************');
        if spepVer < 38.35
            error('Cannot quantify PCASL data for this spep version - header info missing');
        end
        alpha=0.9;
        fmri.func(cntRun).params=parseAFNIhead(fmri.func(cntRun).params, funcbrikLoc,{'pwpcasl'});
        pwpcasl = fmri.func(cntRun).params.pwpcasl./1000000;  % in s
        for nn=1:sd3(3)
            w                 = TI2m(nn) - pwpcasl;      % pwpcasl needs to be in seconds here!
            expTerm           = (exp(-w/T1blood))-(exp(-TI2m(nn)/T1blood));
            CBFraw(:,:,nn,:)  = (ASLdata(:,:,nn,:)./(2.*alpha.*M0blood.*T1blood.*expTerm)).*6000;
        end

    otherwise
        error('**Tagging scheme not recognized!!**');
end

% fill in fmri structure
fmri.func(cntRun).data.CBFraw{1} = CBFraw;
fmri.func(cntRun).data.CBF{1}    = mean(CBFraw,4);


