function fmri=checkCBFcalib(fmri, calibCnt, cntRun)
%usage: fmri=checkCBFcalib(fmri, calibCnt, cntRun)
%
% Performs checks on ASL/CSF/MinCon acquisition, including scan parameters
% and prescan parameters
%
% Input:
%    fmri     - main structure used to get processing flags and data
%    calibCnt - calib run number
%    cntRun   - run number
%
% Output:
%    fmri - fmri.calib.CSF.params
%           fmri.calib.minCon.params
%
% Notes:
%    - For runs to be calibrated into absolute CBF units, a series of checks
%      are made on the ASL acquisition (e.g. use of post-sat, quipps, etc)
%      and CSF/min con acquisitions (prescan values, TR/TE, etc)
%    - Program also reads in calib params so that the R1 scaling is available
%      for CBF scaling in processCBF.
%    - Not yet implemented for doRot data (-do 3dNotes get transfered to new briks?)
%
% Copyright (c) 2006 CUPID
% Author: jperthen
% History:
%  $Id: checkCBFcalib.m,v 1.8 2006/12/15 19:37:43 jperthen Exp $

% ruler --1234567890123456789012345678901234567890123456789012345678901234567890

% turn warning on
Wstatus = warning('query');
warning on;

study       = [fmri.dir.data,'/',fmri.study.name,'/'];
minConFlag  = isfield(fmri.calib(calibCnt).minCon,'name');


% if isfield(fmri,'anat')
%     doRot       = fmri.anat(1).params.doRot;
% else
%     doRot=0;
% end

% if doRot
%     CSFname     = [fmri.calib(calibCnt).CSF.name(1:end-5), '_rot+orig']
%     CSFbrikLoc  = [study,'processed/',CSFname];
%     MCname      = [fmri.calib(calibCnt).minCon.name(1:end-5),'_rot+orig']
%     MCbrikLoc   = [study,'processed/',MCname];
%     error('checkCBFcalib function not yet implemented for doRot!! Talk to Jo!!');
% else
CSFname     = fmri.calib(calibCnt).CSF.name;
CSFbrikLoc  = [study,CSFname];
MCname      = fmri.calib(calibCnt).minCon.name;
MCbrikLoc   = [study,MCname];
funcName    = fmri.func(cntRun).name(1,:);
funcbrikLoc = [study,funcName];
% end

% Check pulseSeq version used
if strcmp(fmri.func(cntRun).params.pulseSeq, 'NaN')
    error('params.pulseSeq info required for CBF calib checking!');
end
spepVer = str2num(fmri.func(cntRun).params.pulseSeq(1,5:end))



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ERROR CHECKS ON ASL ACQUISITION PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% read in info about the func scans
if spepVer > 37.95         % have R1, R2 etc info plus BGS info
    readIn = {'tag','q','mpsR1','mpsR2','mpsTG','apsR1','apsR2',...
        'apsTG','bgs_on','bgs_spatial','psat_on', 'postsat_on'};
else        % may fail for v old spep/gr versions - will need to gr briks again
    readIn = {'tag','q'};
end
fmri.func(cntRun).params=parseAFNIhead(fmri.func(cntRun).params, funcbrikLoc,readIn);

% CHECK: FAIR or PICORE must use Q2
if (fmri.func(cntRun).params.tag == 2 | fmri.func(cntRun).params.tag == 3) && ...
        (fmri.func(cntRun).params.q ~= 2)
    error('FAIR/PICORE data not acquired using QUIPSS II!');
end

% CHECK: VSASL or PCASL must not use Q2
if (fmri.func(cntRun).params.tag == 4 | fmri.func(cntRun).params.tag == 9) && ...
        (fmri.func(cntRun).params.q ~= 0)
    error('VSASL/PCASL data acquired using QUIPSS II!');
end

% CHECK: PQ2 has no post-sat; BGS with PICORE/PCASL must be spatial; VSASL must have post-sat
if spepVer > 37.95
    if (fmri.func(cntRun).params.tag == 2) && (fmri.func(cntRun).params.postsat_on ==1)
        error('PICORE Q2 data acquired using post-sat!');
    end
    if (fmri.func(cntRun).params.bgs_on == 1) && (fmri.func(cntRun).params.bgs_spatial ==0) &&...
            (fmri.func(cntRun).params.tag == 2  | fmri.func(cntRun).params.tag == 9)
        warning('PICORE/PCASL data acquired using BGS but not spatial BGS!');
    end
    if (fmri.func(cntRun).params.tag == 4) && (fmri.func(cntRun).params.postsat_on ==0)
        warning('VSASL data acquired without using post-sat!');
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ERROR CHECKS ON CALIBRATION SCAN ACQUISITION PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if spepVer > 37.95

    % read in info about the csf/mincon scans
    fmri.calib(calibCnt).CSF.params=parseAFNIhead(fmri.calib(calibCnt).CSF.params, CSFbrikLoc,...
        {'reps','TR','TE','nIntlv','nSlice','tag','q','dif',...
        'mpsR1','mpsR2','mpsTG','apsR1','apsR2','apsTG','bgs_on','psat_on','postsat_on'});

    fmri.calib(calibCnt).minCon.params=parseAFNIhead(fmri.calib(calibCnt).minCon.params, MCbrikLoc,...
        {'reps','TR','TE','nIntlv','nSlice','tag','q','dif',...
        'mpsR1','mpsR2','mpsTG','apsR1','apsR2','apsTG','bgs_on','psat_on','postsat_on'});

    % check the acquisition parameters
    if (fmri.calib(calibCnt).CSF.params.bgs_on | fmri.calib(calibCnt).CSF.params.psat_on | fmri.calib(calibCnt).CSF.params.postsat_on)
        error('Check CSF acqusition; should have no BGS/pre-sat/post-sat!');
    end
    if (fmri.calib(calibCnt).minCon.params.bgs_on | fmri.calib(calibCnt).minCon.params.psat_on | fmri.calib(calibCnt).minCon.params.postsat_on)
        error('Check Min Con acqusition; should have no BGS/pre-sat/post-sat!');
    end
    if (fmri.calib(calibCnt).minCon.params.TE ~= 11)
        error('Min Con not acquired at TE 11ms!');
    end
    if (fmri.calib(calibCnt).minCon.params.TR ~= 2)
        error('Min Con not acquired at TR 2s!');
    end
    if (fmri.calib(calibCnt).CSF.params.TE ~= fmri.func(cntRun).params.TE)
        warning('CSF and ASL acquired with different TEs!');
    end

    % check the prescan parameters
    if ~fmri.func(cntRun).params.bgs_on     % prescan params must match
        if (fmri.func(cntRun).params.mpsR1==fmri.calib(calibCnt).CSF.params.mpsR1 &&...
                fmri.func(cntRun).params.mpsR2==fmri.calib(calibCnt).CSF.params.mpsR2 &&...
                fmri.func(cntRun).params.mpsTG==fmri.calib(calibCnt).CSF.params.mpsTG)
        else
            error('CSF data acquired with different prescan values to ASL data!');
        end
    elseif fmri.func(cntRun).params.bgs_on     % R1 prescan params can differ
        if (fmri.func(cntRun).params.mpsR2~=fmri.calib(calibCnt).CSF.params.mpsR2 |...
                fmri.func(cntRun).params.mpsTG~=fmri.calib(calibCnt).CSF.params.mpsTG)
            error('CSF data acquired with different prescan values to ASL data!');
        end
        if (fmri.func(cntRun).params.mpsR1==fmri.calib(calibCnt).CSF.params.mpsR1)
            warning('BGS ASL and CSF acquired with same prescan values; check for overscaling of CSF image!');
        else
            disp(sprintf('R1 difference for BGS ASL and CSF is:  %d',(fmri.func(cntRun).params.mpsR1-fmri.calib(calibCnt).CSF.params.mpsR1)));
        end
    end
end
disp('*****************************');
disp(sprintf('Run %d: CBF checks completed!',cntRun));
disp('*****************************');

% return to original warning status
warning(Wstatus);

