function h = parseAFNIhead(h, brikName, fieldsIn);
%usage: h = parseAFNIhead(h, brikName, fieldsIn);
%e.g. h=parseAFNIhead(h, 'scan1brik+orig', {'TE','TI','slDelay','Rx'});
%
% parse_AFNIhead - parse header info from AFNI brik 'Notes' (created by gr)
%
% Input:
%    h         - include output structure here so as not to overwrite
%                existing fields
%    brikName  - name of Brik to extract details
%    fieldsIn  - CELL ARRAY of fields to fill, e.g. {'pulseSeq','TR'}
%
% Output:
%    h  -  structure with the elements specified in fieldsIn.
%          If 'fieldsIn' is empty, all elements are saved, (existing
%          fields will NOT be overwritten)
%          AVAILABLE FIELDS:
%          gr (gr revision #), pulseSeq (spep revision #), reps, nEcho, TR,
%          TE, nIntlv, nCoil, nPix, FOV, nSlice, slThick, slGap, slDelay,
%          spdir1, spdir2, tag, q, tagThick, tagOffset, TI, voxSize, dif,
%          Rx (slice prescription), voxSize, mpsR1, mpsR2, mpsTG, mpsFREQ
%          (manual prescan values), apsR1, apsR2, apsTG, apsFREQ (auto
%          prescan values)
%
% Notes:
%    1. If fieldsIn is empty, all the possible fields get saved into hh.
%       e.g. hh=parseAFNIhead([],'scan1brik+orig');
%
%    2. Example of how to use in CUPID:
%           Add these lines to defaultFuncStruct:
%               brikName = [fmri.dir.data,'/',fmri.study.name,'/',fmri.func(i).name];
%               fmri.func(i).params = parseAFNIhead(fmri.func(i).params, brikName,...
%                               {'pulseSeq','TE','TI','TR','slDelay','FOV','voxSize',...
%                                  'Rx','mpsR1','mpsR2','apsR1','apsR2'});
%
%           And this line to Setup file (after defaultFuncStruct has been called):
%               fmri.anat(1).params.funcRx  = fmri.func(1).params.Rx;
%
%           For anatomical Rx info:
%               anatName=[fmri.dir.data,'/',fmri.study.name,'/',fmri.anat(1).name];
%               fmri.anat(1).params = parseAFNIhead(fmri.anat(1).params, anatName, {'Rx'});
%
%    3. If new variables are to be added to the AFNI hdr, they must be
%       added to the variable "possFields".
%
% Copyright (c) 2005 CUPID
% Author: jperthen
% History:
%  $Id: parseAFNIhead.m,v 1.19 2006/11/27 19:17:00 jperthen Exp $

% ruler --1234567890123456789012345678901234567890123456789012345678901234567890


% Check first that brik exists
k=strfind(brikName,'.BRIK');
if isempty(k)
   brikNameTest=[brikName '.BRIK'];
else
    brikNameTest=brikName;
end
if exist(brikNameTest)~=2
    error(sprintf('Cannot open %s! Check filename and that file exists!',brikName));
end

possFields = {'gr','pulseSeq','reps','nEcho','TR','TE','nIntlv','nCoil','nPix','FOV','nSlice',...
        'slThick','slGap','slDelay','spdir1','spdir2','tag','q','tagThick','tagOffset','TI',...
        'voxSize','dif','Rx','voxSize','mpsR1','mpsR2','mpsTG','mpsFREQ','apsR1','apsR2',...
        'apsTG','apsFREQ','dda','fsat','bgs_on','bgs_spatial','bgsn','psat_on','postsat_on',...
        'bgsthick','nsfairtagthick','fair_slabsel','oppseq','pwpcasl','sechNSte'};

allFields=0;

if ~exist('fieldsIn') | isempty(fieldsIn)
    allFields=1;
    fieldsIn=possFields;
end

command = ['3dNotes ',brikName,' > AFNINotes'];
system(command);

fid = fopen('AFNINotes');

seqinfo = 0;
rxinfo = 0;pinfo=0;addinfo=0;addinfo2=0;
ss = fgetl(fid);
while ischar(ss)
    if length(ss) > 1
        
        % SEQUENCE INFO
        if strcmp(ss(1:4),'spep');              
            [Sname Svalue] = strread(ss, '%s %f,');
            seqinfo = 1;
            
        % ASL SCAN INFO
        elseif strcmp(ss(1:3),'tag');           
            [ASLname ASLvalue] = strread(ss, '%s %f,');
            
        % ADDITIONAL ASL SCAN INFO (spep038 and above)
        elseif strcmp(ss(1:3),'dda');    
            [ASLname2 ASLvalue2] = strread(ss, '%s %f,');
            addinfo = 1;
            
        % FURTHER ASL SCAN INFO (spep038.4 and above)
        elseif strcmp(ss(1:6),'oppseq');    
            [ASLname3 ASLvalue3] = strread(ss, '%s %f,');
            addinfo2 = 1;
                        
        % GR INFO
        elseif strcmp(ss(1:3),'gr ');           
            try
                [grname grvalue Pname]= strread(ss, '%s %f, %s');
            catch
                try
                    [grname grvalue subver Pname]= strread(ss, '%s %f.%d, %s');   % gr branch version
                catch
                    [grname grvalue] = strread(ss, '%s %f');        % older gr without file name
                end
            end
        
        % MANUAL PRESCAN INFO
        elseif strcmp(ss(1:5),'mpsR1');         
            [MPSname MPSvalue] = strread(ss, '%s %f,');
            pinfo=1;
            
        % AUTO PRESCAN INFO
        elseif strcmp(ss(1:5),'apsR1');        
            [APSname APSvalue] = strread(ss, '%s %f,');
            
        % RX INFO
        elseif strcmp(ss(1:5),'Start');
            [RxStart] = strread(ss, '%s');        
            ss = fgetl(fid);                                
            [RxEnd] = strread(ss, '%s');
            rxinfo = 1;
        end
    end
    ss = fgetl(fid);
end


fclose(fid);

if rxinfo == 0   error(sprintf('No RX information in AFNI header of %s! Use gr15.3 or d2afni!',...
        brikName));end
if seqinfo == 0 warning(sprintf('No sequence info in AFNI header of %s',brikName));end
if pinfo == 0 warning(sprintf('No prescan info in AFNI header of %s',brikName));end

% read parameters into array 'temp'
if seqinfo
    for i=1:length(grvalue)   eval(['temp.',grname{i},'=',num2str(grvalue(i)),';']);    end
    for i=1:length(Svalue)    eval(['temp.',Sname{i},'=',num2str(Svalue(i)),';']);      end
    for i=1:length(ASLvalue)  eval(['temp.',ASLname{i},'=',num2str(ASLvalue(i)),';']);  end
end
if pinfo
    for i=1:length(MPSvalue)  eval(['temp.',MPSname{i},'=',num2str(MPSvalue(i)),';']);   end
    for i=1:length(APSvalue)  eval(['temp.',APSname{i},'=',num2str(APSvalue(i)),';']);   end
end
if addinfo
    for i=1:length(ASLvalue2) eval(['temp.',ASLname2{i},'=',num2str(ASLvalue2(i)),';']); end
end
if addinfo2
    for i=1:length(ASLvalue3) eval(['temp.',ASLname3{i},'=',num2str(ASLvalue3(i)),';']); end
end

temp.Rx{1}=RxStart{2}; temp.Rx{2}=RxEnd{2};
temp.Rx{3}=RxStart{3}; temp.Rx{4}=RxEnd{3};
temp.Rx{5}=RxStart{4}; temp.Rx{6}=RxEnd{4};

% rearrange temp and convert units
if seqinfo
    if temp.q==0  temp.TI=temp.TI2; else  temp.TI=[temp.TI1 temp.TI2]; end
    temp=rmfield(temp,{'TI1', 'TI2'});
    temp.TI=temp.TI*1e-03;              % TI in ms
    if temp.nEcho==2   temp.TE=[temp.TE temp.TE2];  end
    temp=rmfield(temp,{'TE2'});
    temp.TE=temp.TE*1e-03;              % TE in ms
    %if temp.tag==0    temp=rmfield(temp,{'tagThick','tagOffset','TI','q'});  end
    temp.TR=temp.TR*1e-06;              % TR in s
    temp.slDelay=temp.slDelay*1e-03;    % slDelay in ms

    temp.pulseSeq = sprintf('spep0%s', num2str(temp.spep)); temp=rmfield(temp,{'spep'});
    temp.voxSize =[temp.FOV/temp.nPix, temp.FOV/temp.nPix, temp.slThick];

    if temp.TR==0
        error('*** Data acquired with old spep; some parameters not available! Input parameters manually ***');
    end
end

% put selected values into output structure; make error messages sensible
for i=1:length(fieldsIn)
    if isfield(temp, fieldsIn{i})
        eval(['h.',fieldsIn{i},'= temp.',fieldsIn{i},';']);
        
    elseif (~allFields && nnz(strcmp(fieldsIn{i},possFields)))      % field is valid
        disp(sprintf(' ** %s ** is not avaiable for the current BRIK (acquired using old spep)',  fieldsIn{i}));
        
    elseif ~allFields                                               % field is not valid
        error(sprintf(' ** %s ** is not a recognised field name - please check',  fieldsIn{i}));
    end
end


% remove txt file
!rm -f AFNINotes


