function fmri = processCBF(fmri)
%usage: fmri = processCBF(fmri)
% processCBF - generates CBF
%
% Input:
%    fmri  - main structure used to get processing flags and data
%
% Output:
%    fmri - fmri.func.data.CBF
%           fmri.func.data.CBFraw
%           fmri.calib.sensProfile
%           fmri.calib.CSF.data
%           fmri.calib.minCon.data
%    Also filled in during checkCBFcalib:
%           fmri.calib.CSF.params
%           fmri.calib.minCon.params
%
% Need to incorporate TE CSF into CBF calc
%
% Copyright (c) 2005 CUPID
% Author: jperthen
% History:
%  $Id: processCBF.m,v 1.48 2007/02/02 18:39:55 jperthen Exp $

% ruler --1234567890123456789012345678901234567890123456789012345678901234567890

beep on;

calibCnt=NaN;
if fmri.flags.doCBF

    % make 'processed' folder if it does not exist
    brikdir     = [fmri.dir.data,'/',fmri.study.name];
    if exist([brikdir '/processed'])~=7
        mkdir(brikdir,'processed');
    end

    study       = [fmri.dir.data,'/',fmri.study.name,'/'];
    if isfield(fmri,'anat')
        doRot       = fmri.anat(1).params.doRot;
        if doRot
            csfPrefix   = '+reg+rot';
        else
            csfPrefix   = '+reg';
        end
    else
        doRot=0;
    end

    for cntRun=1:length(fmri.func)
        if fmri.func(cntRun).params.doCBF

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % read in CSF and MinCon data
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            % choose appropriate calibration runs
            if ~isfield(fmri.func(cntRun).params,'calibRun')
                fmri.func(cntRun).params.calibRun=1;
            end

            if (cntRun==1 | fmri.func(cntRun).params.calibRun~=calibCnt)
                calibCnt        = fmri.func(cntRun).params.calibRun;
                minConFlag      = isfield(fmri.calib(calibCnt).minCon,'name');

                if doRot
                    CSFname     = [fmri.calib(calibCnt).CSF.name(1:end-5), '_rot+orig']
                    CSFbrikLoc  = [study,'processed/',CSFname];
                else
                    CSFname     = fmri.calib(calibCnt).CSF.name;
                    CSFbrikLoc  = [study,CSFname];
                end

                rawCSF          = getBrik(CSFbrikLoc,0);
                % if CSF data has time dimension, use first image *added ecwong
                if length(size(rawCSF))==4
                    disp('CSF estimated from timecourse data! Using first image for CSF signal!');
                    rawCSF      = squeeze(rawCSF(:,:,:,1));
                end
                % *ecwong

                % save raw CSF data into fmri structure
                fmri.calib(calibCnt).CSF.data = rawCSF;
            end

            % perform checks on CBF data and read in more parameters (e.g. R1, etc)
            % keep outside loop so always runs....
            fmri=checkCBFcalib(fmri,calibCnt,cntRun);

            % Find R1 factor for CSF if BGS used (also keep outside loop)
            R1factor=0.0;
            if isfield(fmri.func(cntRun).params, 'bgs_on')  % later spep versions
                if fmri.func(cntRun).params.bgs_on          % BGS was used
                    R1factorT = (fmri.func(cntRun).params.mpsR1-fmri.calib(calibCnt).CSF.params.mpsR1);
                    if R1factorT>0
                        R1factor=R1factorT;
                    elseif R1factorT<0
                        error('R1 values for CSF and ASL do not make sense!!');
                    end
                end
            end

            if (cntRun==1 | fmri.func(cntRun).params.calibRun~=calibCnt)
                if minConFlag
                    if doRot
                        MCname      = [fmri.calib(calibCnt).minCon.name(1:end-5),'_rot+orig']
                        MCbrikLoc   = [study,'processed/',MCname];
                    else
                        MCname      = fmri.calib(calibCnt).minCon.name;
                        MCbrikLoc   = [study,MCname];
                    end

                    rawMC           = getBrik(MCbrikLoc,0);
                    if isfield(fmri,'anat') & ~isnan(fmri.anat.brain)
                        if doRot & fmri.anat.params.doSeg==3
                            betName = ['processed/' fmri.anat.name(1:end-5) '+reg+rot+betonreg+orig'];
                        elseif doRot & fmri.anat.params.doSeg~=3
                            betName = ['processed/' fmri.anat.name(1:end-5) '+reg+rot+bet+orig'];
                        elseif ~doRot & fmri.anat.params.doSeg==3
                            betName = ['processed/' fmri.anat.name(1:end-5) '+reg+betonreg+orig'];
                        else
                            betName = ['processed/' fmri.anat.name(1:end-5) '+bet+reg+orig'];
                        end

                        eval(['!3dresample -rmode Cu -master ',MCbrikLoc...
                            ' -prefix ',[study,'processed/brain+rs+to+',MCname(1:end-5)],...
                            ' -inset ',[study,betName]]);
                        brain = getBrik([[study,'processed/brain+rs+to+',MCname(1:end-5)],...
                            '+orig'],0,'int16')>40;

                        MCMaskBest  = smoothmask(double(brain));
                    else
                        brain = 0;
                    end

                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    % create sensitivity profiles and apply to CSF and d3 data
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                    disp('sensitivity correction being performed');
                    MCrepsS = ceil((size(rawMC,4))/2);
                    if size(rawMC,4)==1
                        disp('warning! only one repetition of min con image acquired!');
                    end
                    MCave           = mean(rawMC(:,:,:,MCrepsS:end),4);

                    % save minCon data into fmri structure
                    fmri.calib(calibCnt).minCon.data = MCave;

                    pThresh         = 0.3;
                    MCMaskEst       = thremask(MCave,pThresh,1);
                    MCMaskEst       = smoothmask(MCMaskEst);
                    if brain==0
                        MCMask      = MCMaskEst;
                    else
                        MCMask      = MCMaskBest.*MCMaskEst;
                    end
                    sMC             = size(MCave);
                    MCave           = MCave.*MCMask;
                    Wstatus = warning('query');
                    warning off;
                    for dd=1:sMC(end)
                        smMC(:,:,dd)= surfFit(MCave(:,:,dd), 4);     % lorder 4 looks best (but lower in brain, 3 may be better...)
                    end
                    smMC            = smMC./(mean(nonzeros(smMC))); % rough normalisation

                    % set value of sensitivity map outside the mask to one
                    oneStd              = std(nonzeros(smMC));
                    smMC(find(smMC==0)) = 1.0;
                    % set outlying sensitivity map values to one
                    smMC(find(smMC<1-2*oneStd)) = 1.0;
                    smMC(find(smMC>1+2*oneStd)) = 1.0;

                    % save sensitivity profiles into fmri structure
                    fmri.calib(calibCnt).sensProfile = smMC;
                    warning(Wstatus);
                else
                    disp('NO sensitivity correction being performed');
                    smMC   = ones(size(rawCSF));
                    MCMask = ones(size(rawCSF));
                    sMC    = size(rawCSF);
                end

                %correct the CSF images
                sCSF   = size(rawCSF);
                if sCSF(end)~=sMC(end)
                    error('# CSF slices not equal to # MinContrast slices!');
                end
                CSF    = rawCSF.*MCMask;

                corrCSF=zeros(size(rawCSF));
                for kk=1:sCSF(end)
                    corrCSF(:,:,kk) = CSF(:,:,kk)./smMC(:,:,kk);
                end
                corrCSF(isinf(corrCSF)) = 0;
                corrCSF(isnan(corrCSF)) = 0;
            end
            disp(['** Calibrating ' fmri.func(cntRun).name(1,:) ' using '  fmri.calib(calibCnt).minCon.name ' & ' fmri.calib(calibCnt).CSF.name ' **'])

            % correct the d3 timecourse
            d3Data  = fmri.func(cntRun).data.d3{1};
            sd3     = size(d3Data);
            xslices = (sCSF(end)-sd3(3))/2;
            if xslices~=round(xslices)
                error('Calibration slices not as expected!');
            end

            if 0    % JP - remove the masking of the CBF maps
                d3Mask    = MCMask(:,:,xslices+1:xslices+sd3(3));
                d3MaskMat = repmat(d3Mask, [1 1 1 sd3(end)]);
                md3Data   = d3Data.*d3MaskMat;
            end
            md3Data       = d3Data;
            d3smMC        = smMC(:,:,(xslices+1:xslices+sd3(3)));
            d3smMCMat     = repmat(d3smMC, [1 1 1 sd3(end)]);
            corrd3        = md3Data./d3smMCMat;
            corrd3(isinf(corrd3)) = 0;
            corrd3(isnan(corrd3)) = 0;


            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % calculate reference SI value from either WM...
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            if strcmp(fmri.func(cntRun).params.SIref,'WM')
                disp('** Using WM as signal intensity reference for CBF calculation **');
                WMfile   = [fmri.dir.data,'/',fmri.study.name,'/processed/WMvalue.txt'];
                if exist(WMfile, 'file')
                    beep;
                    answer   = input('Do you want to use saved value from WMvalue.txt (y/[n])?', 's');
                    if strcmp(lower(answer), 'y') | strcmp(lower(answer), 'yes')
                        WMvalue=load(WMfile);
                    else
                        answer='n';
                    end
                else
                    answer = 'n';
                end

                if strcmp(answer, 'n')
                    figure;
                    for nn=1:sCSF(end)
                        subplot(1,sCSF(end),nn);
                        imagesc(rot90(corrCSF(:,:,nn),3));axis square; colormap gray;
                        title(sprintf('Reference slice %g', nn));
                    end
                    WMimageNo   = (input(['Choose slice for occipital WM ? : ']));
                    WMimage     = squeeze(corrCSF(:,:,WMimageNo));
                    disp('Draw carefully around region of OCCIPITAL WM');
                    figure;imagesc(rot90(WMimage,3));colormap('gray');axis square;nfig=gcf;
                    [pat]       = roipoly;
                    pat         = rot90(pat,-3);
                    patch       = pat.*WMimage;
                    close(nfig);
                    WMvalue     = mean(patch(find(patch~=0)));
                    % show chosen voxels
                    overLay2(WMimage,pat,0.4,'gray','autumn',1,3);
                    title(sprintf('Selected WM voxels; mean value=%0.1f', WMvalue'));
                    % save WM value for later use
                    fid         = fopen(WMfile,'w');
                    fprintf(fid,'%f\n',WMvalue);
                    fclose(fid);
                end

                SIrefValue      = WMvalue;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % ... else calculate reference SI value from CSF
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            elseif strcmp(fmri.func(cntRun).params.SIref,'CSF')
                disp('** Using CSF as signal intensity reference for CBF calculation **');
                CSFfile   = [fmri.dir.data,'/',fmri.study.name,'/processed/',CSFname,'CSFvalue.txt'];
                if exist(CSFfile, 'file')
                    beep;
                    answer   = input(['Do you want to use saved value from ',CSFname,'CSFvalue.txt (y/[n])?'], 's');
                    if strcmp(lower(answer), 'y')
                        CSFvalue = load(CSFfile);
                    else
                        answer='n';
                    end
                else
                    answer = 'n';
                end

                if strcmp(answer, 'n')
                    % IF anat exists, get CSF value using CSF PV map to locate CSF voxels
                    if isfield(fmri,'anat')
                        if exist([brikdir '/processed/' fmri.anat.name(1:end-5) csfPrefix '+csf+pv+orig.BRIK' ])==2

                            % resample anat CSF PV data to CSF brik and read in
                            eval(['!3dresample -rmode ''Li'' -prefix '...
                                brikdir '/processed/' fmri.anat.name(1:end-5) csfPrefix '+csf+pv+rs2csf' num2str(calibCnt) ' -master '...
                                CSFbrikLoc ' -inset '...
                                brikdir '/processed/' fmri.anat.name(1:end-5) csfPrefix '+csf+pv+orig']);
                            csfVol=getBrik([brikdir '/processed/' fmri.anat.name(1:end-5) csfPrefix '+csf+pv+rs2csf' num2str(calibCnt) '+orig'],0,'float');

                            % get user to specifiy where the ventricles are by drawing on each slice
                            nSl=size(corrCSF,3);
                            corrCSFDraw=zeros(size(corrCSF));
                            for currSl=1:nSl
                                corrCSFDraw(:,:,currSl)=rot90(corrCSF(:,:,currSl),3);
                            end
                            disp('*************Using anatomical to find CSF voxels***************');
                            disp('***************Draw roughly around ventricles******************');
                            disp('**Double click in image if no ventricles on the current slice**');
                            Ventroi=drawRoiS(corrCSFDraw,-3,1);         % rotate the ROI back again

                            % get CSF value by taking highest 5 pixels within the WHOLE Ventricular ROI - use as a
                            % sanity check for the new method below. This value is never used as the CSFvalue
                            patch              = Ventroi.*corrCSF;
                            [CSFsort, CSFind]  = sort(patch(:));
                            CSFvalue1          = mean(CSFsort(end-4:end));
                            %overLay2(csfVol, Ventroi, [],[],[],4);
                            % use all voxels with csf pv>0.95, remove the lowest (mean-1std pixels) before averaging
                            csfMask=(csfVol>0.95);
                            csfVolpix=csfMask.*Ventroi;
                            [R,C,V] = find(csfVolpix.*corrCSF);
                            keptVals=V(V>(mean(V)-std(V)));
                            CSFvalue2=mean(keptVals);

                            disp(sprintf('Found %d pixels >0.95 CSF, keeping %d',nnz(csfVolpix), length(keptVals)));
                            if nnz(csfVolpix)<5
                                disp('Very few pixels!!');
                            end

                            % display the chosen voxels
                            VentroiT=Ventroi;
                            VentroiT(1,1,:)=4;  % for proper scaling of overlays
                            VentroiT(find(csfVolpix))=4;
                            overLay2(corrCSF,VentroiT,0.5,'gray','autumn',4,3);
                            title(sprintf('Selected CSF voxels (yellow) within drawn ROI (red)\n(values < mean-1std will be removed before averaging)'), 'FontSize', 14);

                            % check the CSF value makes sense
                            CSFdiff=((CSFvalue2-CSFvalue1)/CSFvalue2).*100;
                            disp(sprintf('percent diff between two methods is: %1.1f',CSFdiff));
                            if CSFdiff<20
                                CSFvalue=CSFvalue2;
                            else
                                ans   = input('There is a large difference between the two CSF methods!! Do you want to continue with the anatomical value? (y/n)?', 's');
                                if strcmp(lower(ans), 'y')
                                    CSFvalue=CSFvalue2;
                                else
                                    error('CSF value not consistent!');
                                end
                            end
                        else
                            disp('*****************************************');
                            disp(sprintf('Could not find CSF partial volume map (should have been created by processAnat, \nusing fmri.anat.params.doSeg=3) - not using anatomical to find CSF pixels!'));
                            disp('*****************************************');
                        end

                    else       % no anat - use old top-5 method

                        %find range for displaying corrCSF data, KR
                        [h b]          = hist(corrCSF(find(MCMask)),round(range(corrCSF(find(MCMask)))/400));
                        h(1)           = 0;
                        [d ind]        = max(h);
                        hout           = h(ind:end);
                        hin            = h(1:ind);
                        vclip          = 100;           %clip at 100 voxels
                        [houtd ihoutd] = min(abs(hout-vclip));
                        [hind ihind]   = min(abs(fliplr(hin)-vclip));
                        imsRange       = [b(ind-ihind) b(ind+ihoutd)];
                        ims(corrCSF,5,imsRange,1,0,1);colormap gray

                        CSFimageNo     = (input(['Choose slice for CSF (beware partial-volumed slices!) ? : ']));
                        CSFimage       = squeeze(corrCSF(:,:,CSFimageNo));
                        disp('Draw roughly around region of CSF');
                        ims(CSFimage,1,imsRange,1,0,1);colormap gray;nfig=gcf;
                        [pat]          = roipoly;
                        pat            = rot90(pat,-3);
                        patch          = pat.*CSFimage;
                        close(nfig);
                        [CSFsort, CSFind] = sort(patch(:));
                        CSFvalue          = mean(CSFsort(end-4:end));
                        % show chosen voxels
                        CSFimageDisp      = CSFimage;
                        CSFimageDisp(CSFind(end-4:end)) = 2.5*CSFimage(CSFind(end-4:end));
                        overLay2(CSFimageDisp,pat,0.2,'gray','autumn',1,3);
                        title(sprintf('Selected CSF voxels within drawn ROI; mean value=%0.1f', CSFvalue'));
                    end

                    % save CSF value for later use
                    fid = fopen(CSFfile,'w');
                    fprintf(fid,'%f\n',CSFvalue);
                    fclose(fid);
                end

                SIrefValue       = CSFvalue;
            else
                error('Tissue type for SI reference is not specified!');
            end

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % calculate a CBF value
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            fmri=CBFcalc(fmri, SIrefValue, cntRun, corrd3, R1factor);

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % display and save into fmri structure
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            figure('Name','CBF (ml/100g tissue/min)');
            if (sd3(3)>5)
                numcol=ceil(sqrt(sd3(3)));
                numrow=ceil(sd3(3)/numcol);
            else
                numcol=sd3(3);
                numrow=1;
            end
            smontage(fmri.func(cntRun).data.CBF{1},numrow,numcol,[0 100]);axis off;
            title(sprintf('CBF data, run %d', cntRun));

        end     %fmri.func(cntRun).params.doCBF
    end     %cntRun
end     %if(fmri.flags.doCBF)
